@@ -8,7 +8,7 @@
    Copyright (c) The Exim Maintainers 2016
 */
 
-#ifdef EXPERIMENTAL_SPF
+#ifdef SUPPORT_SPF
 
 /* Yes, we do have ns_type. spf.h redefines it if we don't set this. Doh */
 #ifndef HAVE_NS_TYPE
@@ -25,8 +25,8 @@ typedef struct spf_result_id {
 } spf_result_id;
 
 /* prototypes */
-int spf_init(uschar *,uschar *);
-int spf_process(const uschar **, uschar *, int);
+BOOL spf_init(uschar *,uschar *);
+int  spf_process(const uschar **, uschar *, int);
 
 #define SPF_PROCESS_NORMAL  0
 #define SPF_PROCESS_GUESS   1
